package com.jh.boot.template.customizable;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


@Service
public class CustomizableLabelsService {

	private CustomizableLabelsRepository repo;
	
	private Map<String, CustomizableTemplateLabels> templateLabelsCache = new HashMap<>();
	
	private boolean useCache;
	

	@Transactional
	public Map<String, CustomizableTemplateLabel> getCustomizationTemplateLabels(String templateCode, Long unitId, String defaultLanguage) {
		TemplateLabels templateLabels = getTemplateDefaultLabels(templateCode);
		List<CustomizableTemplateLabel> labels = repo.findCustomizedTemplateLabels(templateCode, unitId);
		
		Map<String, CustomizableTemplateLabel> defaultMap = templateLabels.getLanguages().get(defaultLanguage);
		if (defaultMap == null) {
			defaultMap = templateLabels.getFallbackLabels();
		}
		if (defaultMap == null) {
			return null;
		}
		
		Map<String, CustomizableTemplateLabel> result = new HashMap<>();
		for (String code : templateLabels.getCodes()) {
			CustomizableTemplateLabel label = defaultMap.get(code);
			
			CustomizableTemplateLabel newLabel = new CustomizableTemplateLabel();
			newLabel.setCode(code);
			newLabel.setLabel(label.getLabel());
			newLabel.setDefaultLabel(label.getLabel());
			newLabel.setPosition(label.getPosition());
			
			result.put(code, newLabel);
		} 
		
		for (CustomizableTemplateLabel label : labels) {
			CustomizableTemplateLabel defaultLabel = result.get(label.getCode());
			if (defaultLabel != null) {
				label.setDefaultLabel(defaultLabel.getLabel());
			}
			result.put(defaultLanguage, label);
		}
		
		return result;
	}	
	
	@Transactional
	public Map<String, String> getTranslatedTemplateLabels(String templateCode, Long unitId, String defaultLanguage) {
		TemplateLabels templateLabels = getTemplateDefaultLabels(templateCode);
		List<CustomizableTemplateLabel> labels = repo.findCustomizedTemplateLabels(templateCode, unitId);
		
		Map<String, CustomizableTemplateLabel> defaultMap = templateLabels.getLanguages().get(defaultLanguage);
		if (defaultMap == null) {
			defaultMap = templateLabels.getFallbackLabels();
		}
		if (defaultMap == null) {
			return null;
		}
		
		Map<String, String> result = new HashMap<>();
		for (String code : templateLabels.getCodes()) {
			CustomizableTemplateLabel label = defaultMap.get(code);
			if (label != null) {
				result.put(code, label.getLabel());
			}
		}
		
		for (CustomizableTemplateLabel label : labels) {
			result.put(label.getCode(), label.getLabel());
		}
		
		return result;
	}
	
	@Transactional
	public List<CustomizableTemplateLabel> fetchLabels(String templateCode, Long unitId) {
		return repo.findCustomizedTemplateLabels(templateCode, unitId);
	}
	
	
	@Transactional
	public TemplateLabels getTemplateDefaultLabels(String templateCode) {
		CustomizableTemplateLabels labels = useCache ? templateLabelsCache.get(templateCode) : null;
		if (labels != null) {
			return labels;
		}
		
		
		Set<String> codes = new LinkedHashSet<>();
		Map<String, Map<String, CustomizableTemplateLabel>> maps = new HashMap<>();
		Map<String, CustomizableTemplateLabel> fallbackMap = new HashMap<>();

		List<CustomizableTemplateLabel> defaultLabels = repo.findDefaultTemplateLabels(templateCode);

		
		//keep order
		for (CustomizableTemplateLabel ctl : defaultLabels) {
			codes.add(ctl.getCode());
		}
		
		//map labels without order
		for (CustomizableTemplateLabel ctl : defaultLabels) {
			Map<String, CustomizableTemplateLabel> map = maps.get(ctl.getLanguageCode());
			if (map == null) {
				maps.put(ctl.getLanguageCode(), map = new HashMap<>());
			}
			map.put(ctl.getCode(), ctl);
			if ("en".equalsIgnoreCase(ctl.getLanguageCode()) || !fallbackMap.containsKey(ctl.getCode())) {
				fallbackMap.put(ctl.getCode(), ctl);
			}
		}

		labels = new CustomizableTemplateLabels();
		labels.codes = codes;
		labels.languages = new HashMap<>();
		
		for (String lang : maps.keySet()) {
			labels.languages.put(lang, new HashMap<>());
		}
		
		// align maps
		for (Entry<String, Map<String, CustomizableTemplateLabel>> entry : maps.entrySet()) {
			Map<String, CustomizableTemplateLabel> langMap = labels.languages.get(entry.getKey());
			for (String code : codes) {
				CustomizableTemplateLabel ctl = entry.getValue().get(code);
				if (ctl == null) {
					ctl = fallbackMap.get(code);
				}
				if (ctl == null) {
					continue;
				}
				
				langMap.put(code, ctl);
			}
		}
		
		if (useCache) {
			templateLabelsCache.put(templateCode, labels);
		}
		return labels;
	}
	
	
	@Transactional
	public void saveCustomizedLabels(Collection<CustomizableTemplateLabel> labels) {
		repo.saveCustomizedLabels(labels);
	}

//
//	@Transactional
//	public void saveCustomizedLabels(Long unitId, String templateCode, Map<String, String> labels) {
//		Map<String, String> map = new HashMap<>(labels);
//		List<CustomizableTemplateLabel> list = repo.findCustomizedTemplateLabels(templateCode, unitId);
//		
//		for (CustomizableTemplateLabel label : list) {
//			String s = map.get(label.getCode());
//			if (s == null) {
//				repo.delete(label);
//			} else {
//				label.setLabel(s);
//				repo.save(label);
//				map.remove(label.getCode());
//			}
//		}
//		
//		for (Entry<String, String> e : map.entrySet()) {
//			CustomizableTemplateLabel label = new CustomizableTemplateLabel();
//			label.setUnitId(unitId);
//			label.setTemplate(new Template(templateCode));
//			label.setCode(e.getKey());
//			label.setLabel(e.getValue());
//			repo.save(label);
//		}
//	}


	@Autowired(required = false)
	public void setRepo(CustomizableLabelsRepository repo) {
		this.repo = repo;
	}
	

	@Value("${global.caching:true}")
	public void setUseCache(boolean useCache) {
		this.useCache = useCache;
	}	
	
	
	protected class CustomizableTemplateLabels implements TemplateLabels {

		Set<String> codes = new LinkedHashSet<>();

		Map<String, Map<String, CustomizableTemplateLabel>> languages = new HashMap<>();

		Map<String, CustomizableTemplateLabel> fallbackLabels = new HashMap<>();
		
		@Override
		public Set<String> getCodes() {
			return codes;
		}

		@Override
		public Map<String, Map<String, CustomizableTemplateLabel>> getLanguages() {
			return languages;
		}

		@Override
		public Map<String, CustomizableTemplateLabel> getFallbackLabels() {
			return fallbackLabels;
		}

	}
	
}
